﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Drawing.Design;

namespace Infragistics.Controls
{
    /// <summary>
    /// Object that is showed in PropertyGrid as ReadOnly
    /// </summary>
    public class ReadOnlyObject : ICustomTypeDescriptor
    {
        private object obj;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ReadOnlyObject"/> class.
        /// </summary>
        /// <param name="obj">The obj.</param>
        public ReadOnlyObject(object obj)
        {
            this.obj = obj;
        }

        #region ICustomTypeDescriptor Members

        /// <summary>
        /// Returns a type converter for this object.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.ComponentModel.TypeConverter"/> that is the converter for this object, or
        /// <see langword="null "/>if there is no <see cref="T:System.ComponentModel.TypeConverter"/> for this
        /// object.
        /// </returns>
        public TypeConverter GetConverter()
        {
            return TypeDescriptor.GetConverter(this.obj, true);
        }

        /// <summary>
        /// Returns the events for this instance of a component using the attribute array as a
        /// filter.
        /// </summary>
        /// <param name="attributes">An array of type <see cref="T:System.Attribute"/> that is used as a filter.</param>
        /// <returns>
        /// An <see cref="T:System.ComponentModel.EventDescriptorCollection"/> that
        /// represents
        /// the events for this component instance that match the given set of
        /// attributes.
        /// </returns>
        public EventDescriptorCollection GetEvents(Attribute[] attributes)
        {
            return TypeDescriptor.GetEvents(this.obj, attributes, true);
        }

        /// <summary>
        /// Returns the events for this instance of a component.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.ComponentModel.EventDescriptorCollection"/> that
        /// represents the events for this component instance.
        /// </returns>
        public EventDescriptorCollection GetEvents()
        {
            return TypeDescriptor.GetEvents(this.obj, true);
        }

        /// <summary>
        /// Returns the name of this object.
        /// </summary>
        /// <returns>
        /// The name of the object, or <see langword="null "/> if object does not have a name.
        /// </returns>
        public string GetComponentName()
        {
            return TypeDescriptor.GetComponentName(this.obj, true);
        }

        /// <summary>
        /// Returns the object that this value is a member of.
        /// </summary>
        /// <param name="pd">A <see cref="T:System.ComponentModel.PropertyDescriptor"/> that represents the property whose owner is to be found.</param>
        /// <returns>
        /// An <see cref="T:System.Object"/> that represents
        /// the owner of the specified property.
        /// </returns>
        public object GetPropertyOwner(PropertyDescriptor pd)
        {
            return this.obj;
        }

        /// <summary>
        /// Returns a collection of type <see cref="T:System.Attribute"/>
        /// for this object.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.ComponentModel.AttributeCollection"/> with the attributes for this object.
        /// </returns>
        public AttributeCollection GetAttributes()
        {
            return TypeDescriptor.GetAttributes(this.obj, true);
        }

        /// <summary>
        /// Returns the properties for this instance of a component using the attribute array as a filter.
        /// </summary>
        /// <param name="attributes">An array of type <see cref="T:System.Attribute"/> that is used as a filter.</param>
        /// <returns>
        /// An array of type <see cref="T:System.Attribute"/> that represents the properties for this
        /// component instance that match the given set of attributes.
        /// </returns>
        public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {
            return GetProperties();
        }

        /// <summary>
        /// Returns the properties for this instance of a component.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.ComponentModel.PropertyDescriptorCollection"/> that
        /// represents
        /// the properties for this component instance.
        /// </returns>
        public PropertyDescriptorCollection GetProperties()
        {
            PropertyDescriptorCollection coll;

            if (this.obj is ICustomTypeDescriptor)
                coll = (this.obj as ICustomTypeDescriptor).GetProperties();
            else
                coll = TypeDescriptor.GetProperties(this.obj, true);

            PropertyDescriptorCollection outcoll = new PropertyDescriptorCollection(null);

            foreach (PropertyDescriptor desc in coll)
            {
                ReadOnlyPropertyDescriptor pd = new ReadOnlyPropertyDescriptor(desc);
                outcoll.Add(pd);
            }

            return outcoll;
        }

        /// <summary>
        /// Returns an editor of the specified type for this object.
        /// </summary>
        /// <param name="editorBaseType">A <see cref="T:System.Type"/> that represents the editor for this object.</param>
        /// <returns>
        /// An <see cref="T:System.Object"/> of the specified type that is the editor for this object,
        /// or <see langword="null "/>if the
        /// editor cannot be found.
        /// </returns>
        public object GetEditor(Type editorBaseType)
        {
            return new UITypeEditor();
        }

        /// <summary>
        /// Returns the default property for this object.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.ComponentModel.PropertyDescriptor"/> that
        /// represents the default property for this object, or <see langword="null "/>if
        /// this object does not have properties.
        /// </returns>
        public PropertyDescriptor GetDefaultProperty()
        {
            return TypeDescriptor.GetDefaultProperty(this.obj, true);
        }

        /// <summary>
        /// Returns the default event for this object.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.ComponentModel.EventDescriptor"/> that represents the default event for
        /// this object, or <see langword="null "/>if this object does not have events.
        /// </returns>
        public EventDescriptor GetDefaultEvent()
        {
            return TypeDescriptor.GetDefaultEvent(this.obj, true);
        }

        /// <summary>
        /// Returns the class name of this object.
        /// </summary>
        /// <returns>
        /// The class name of the object, or <see langword="null "/> if the class does not have a name.
        /// </returns>
        public string GetClassName()
        {
            return TypeDescriptor.GetClassName(this.obj, true);
        }

        #endregion

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            if (obj != null)
            {
                return obj.ToString();
            }

            return string.Empty;
        }
    }
}
